@@ -12,7 +12,7 @@
            ## supplement for LevenbergM
            beta_eta,
            ## supplement for ! GLMM (??)
-           u_h, v_h, # for_init_z_args, 
+           u_h, v_h,  
            #
            trace=FALSE,
            stylefn=.spaMM.data$options$stylefns$vloop,
@@ -35,14 +35,7 @@
     pot_tol <- processed$spaMM_tol$v_pot_tol * looseness
     d_relV_tol <- processed$spaMM_tol$d_relV_tol * looseness
     constant_zAug_args <- list(n_u_h=n_u_h, nobs=nobs, pforpv=pforpv, y=y, off=off, ZAL=ZAL, processed=processed)
-    # if ( ! GLMMbool) {
-    #   constant_init_z_args <- c(list(lcrandfamfam=lcrandfamfam, nobs=nobs, lambda_est=lambda_est, ZAL=ZAL),  
-    #                             # fit_as_ZX args specific for ! GLMM:
-    #                             for_init_z_args,
-    #                             #
-    #                             mget(c("cum_n_u_h","rand.families"),envir=processed))
-    # } 
-    
+
     ##### initial sXaug
     ZAL_scaling <- 1  ## TAG: scaling for spprec
     eta  <- off + drop(processed$AUGI0_ZX$X.pv %*% beta_eta) + drop(ZAL %id*% v_h) 
