@@ -31,7 +31,7 @@ spaMM_boot <- local({
     # If the simuland has (say) arguments y, what=NULL, lrt, ...   , we should not have lrt in the dots. Since the dots are not directly manipulable
     # we have to convert them to a list, and ultimately to use do.call()
     control.foreach$.combine <- "rbind"
-    wrap_parallel <- get(.spaMM.data$options$wrap_parallel, asNamespace("spaMM"), inherits=FALSE)
+    wrap_parallel <- get(.spaMM.data$options$wrap_parallel, asNamespace("spaMM"), inherits=FALSE) # dopar
     bootreps <- wrap_parallel(newresp = newy_s, nb_cores = nb_cores,fn = simuland, fit_env = fit_env, 
                        control=control.foreach, debug.=debug., pretest_cores = .pretest_fn_on_cores, 
                       showpbar = showpbar, ...) 
@@ -108,7 +108,7 @@ spaMM_boot <- local({
       if (is.null(type_user)) {
         if (.inRstudio(silent=TRUE)) {
           cluster_args$type <- "PSOCK"
-        } else cluster_args$type <- "PSOCK" # default is socket cluster in all cases
+        } else cluster_args$type <- "PSOCK" # a way to emphasize that default is socket cluster in all cases (the doc says so)
       } else if (type_user=="FORK" && .inRstudio(silent=TRUE)) {
         message('cluster_args$type=="FORK" not feasible when R is called from an Rstudio session.')
         cluster_args$type <- "PSOCK"
@@ -126,7 +126,7 @@ dopar <- local({
            fit_env, control=list(), cluster_args=NULL,
            debug.=FALSE, iseed=NULL, showpbar=eval(spaMM.getOption("barstyle")),
            pretest_cores=NULL,
-           ... # passed to fn
+           ... # passed to fn... unless captured by pbapply (in which case 'simplify' may have a distinct effect).
   ) {
     if (is.list(fit_env)) fit_env <- list2env(fit_env)
     cluster_args <- .set_cluster_type(cluster_args, nb_cores) # PSOCK vs FORK
@@ -143,8 +143,8 @@ dopar <- local({
       }
       if (cluster_args$type=="FORK") {
         if (is.null(mc.silent <- control$mc.silent)) mc.silent <- TRUE 
-        has_progressr <- ("package:progressr" %in% search())
-        seq_nr <- seq_len(ncol(newresp))
+        has_progressr <- ("progressr" %in% loadedNamespaces())
+        seq_nr <- seq_len(nsim)
         if (has_progressr) {
           # progressor is the only progress function that 'works' with mclapply
           # although not with load-balancing (mc.preschedule=FALSE)
@@ -152,7 +152,7 @@ dopar <- local({
           prog_fn <- get("progressor", asNamespace("progressr"), inherits=FALSE) # syntax for using an undeclared package (cf stats:::confint.glm)
           with_fn <- get("with_progress", asNamespace("progressr"), inherits=FALSE) # syntax for using an undeclared package (cf stats:::confint.glm)
           with_fn({
-            p <- prog_fn(steps=ceiling(ncol(newresp)/nb_cores))
+            p <- prog_fn(steps=ceiling(nsim/nb_cores))
             p_fn <- function(it, ...) { # it OK for mclapply... not for apply on a matrix
               res <- fn(newresp[,it], ...)
               p() # p() call necessary for actual progress report 
@@ -176,7 +176,8 @@ dopar <- local({
         cl <- do.call(parallel::makeCluster, cluster_args) # note that _this_ line would make sense for fork clusters too. BUT
         # ... the foreach = dot args combination may not work for FORK type. Only pbapply would work with makeCluster+FORK, 
         # but pbmcapply is a better way to get a pb one a fork cluster as [pb]mclapply have better load balancing than pbapply. 
-        has_doSNOW <- ("package:doSNOW" %in% search())
+        # has_doSNOW <- ("package:doSNOW" %in% search()) # result of library()
+        has_doSNOW <- ("doSNOW" %in% loadedNamespaces()) # result of library() or requireNamespace()
         if (has_doSNOW) {
           # loading (?) the namespace of 'snow' changes the *parent* RNG state (as well as sons' ones)! so we save and restore it 
           R.seed <- get(".Random.seed", envir = .GlobalEnv) # save parent RNG state
@@ -185,23 +186,23 @@ dopar <- local({
           assign(".Random.seed", R.seed, envir = .GlobalEnv) # restore parent RNG state
           if ( ! is.null(iseed) ) parallel::clusterSetRNGStream(cl = cl, iseed) 
           #
-          if (cluster_args$type == "PSOCK") {
+          # if (cluster_args$type == "PSOCK") {
             if (is.environment(fit_env)) parallel::clusterExport(cl=cl, varlist=ls(fit_env), envir=fit_env) 
             pb_char <- "P"
-          } else pb_char <- "F"
+          # } else pb_char <- "F"
           # A first foreach_blob for a first dopar before defining the progress bar (otherwise we see a progress bar on this dopar)
           i <- NULL ## otherwise R CMD check complains that no visible binding for global variable 'i' (in expression newy_s[,i])
           foreach_blob <- foreach::foreach(i=1:nb_cores)
-          if (cluster_args$type == "PSOCK") {
+          #if (cluster_args$type == "PSOCK") {
             abyss <- foreach::`%dopar%`(foreach_blob, Sys.setenv(LANG = "en")) # before setting the progress bar...
             if (is.function(pretest_cores)) pretest_cores(fn, cl)
-          }
+          #}
           # define the progress bar:
           barstyle <- eval(spaMM.getOption("barstyle"))
           progrbar_setup <- .set_progrbar(max = nsim, style = barstyle, char=pb_char)
           # :where opts are needed to define a second foreach_blob
           foreach_args <- list( 
-            i = 1:ncol(newresp), 
+            i = 1:nsim, 
             .combine = "cbind", 
             .inorder = TRUE, .packages = "spaMM", 
             .errorhandling = "remove", ## use "pass" to see problems
@@ -242,7 +243,6 @@ dopar <- local({
           #
           if (showpbar) close(progrbar_setup$pb)
         } else { # no doSNOW
-          # in that case, ## We will use pbapply, with argument cl=cl; a direct call to foreach would require doParallel::registerDoParallel(cl)
           if ( ! doSNOW_warned) {
             message("If the 'doSNOW' package were attached, better load-balancing might be possible.")
             doSNOW_warned <<- TRUE
@@ -256,6 +256,16 @@ dopar <- local({
                                 function(packages) {for (p in packages) library(p, character.only = TRUE)}, 
                                 packages2export)
           if (is.environment(fit_env)) try(parallel::clusterExport(cl=cl, varlist=ls(fit_env), envir=fit_env)) 
+          
+          # in that case, ## We will use pbapply, with argument cl=cl; 
+          # Given no doSNOW, a direct call to foreach would require doParallel::registerDoParallel(cl)
+          # or doFuture::registerDoFuture(). 
+          # There are fake solutions suggesting a progress bar can be set up with doParallel, 
+          # but it actually progresses only after all the processes have been run
+          # (proposed examples have too short processes for this to be apparent).
+          # So ultimately... we need doFuture, ( => see distinct wrapper).
+          # and current we use pbapply with an ad hoc treatment for combining the results 
+          
           if (is.function(pretest_cores)) pretest_cores(fn, cl)
           if (showpbar) {
             pbopt <- pboptions(nout=min(100L,2L*nsim),type="timer",char=pb_char) 
@@ -266,25 +276,69 @@ dopar <- local({
           pboptions(pbopt)
           if (inherits(bootreps,"try-error") &&
               grep("could not find",(condmess <- conditionMessage(attr(bootreps,"condition"))))
-              ) {
+          ) {
             firstpb <- strsplit(condmess,"\"")[[1]][2]
             cat(crayon::bold(paste0(
               "Hmmm. It looks like some variables were not passed to the parallel processes.\n",
               "Maybe add    ",firstpb," = ",firstpb,"   to spaMM_boot()'s 'fit_env' argument?\n"
             )))
           }
-          if (identical(control$.combine,"rbind")) bootreps <- t(bootreps)
+          if (identical(control$.combine,"rbind")) bootreps <- t(bootreps) # this means the pbapply version handles cbind or rbind but not other 
         } # has_doSNOW ... else
       } # FORK ... else
       if ( ! is.null(iseed) ) do.call("RNGkind", as.list(ori)) # reste to state pre-parallel computation
     } else { ## nb_cores=1L
       pb_char <- "s"
-      if (showpbar) {
-        pbopt <- pboptions(nout=min(100L,2L*nsim),type="timer",char=pb_char) 
-      } else pbopt <- pboptions(type="none") 
-      bootreps <- pbapply(X=newresp,MARGIN = 2L,FUN = fn, cl=NULL, ...)
-      pboptions(pbopt)
-      if (identical(control$.combine,"rbind")) bootreps <- t(bootreps)
+      if (FALSE) {
+        # :where opts are needed to define a second foreach_blob
+        foreach_args <- list( 
+          i = 1:ncol(newresp), 
+          .combine = "cbind", 
+          .inorder = TRUE, .packages = "spaMM", 
+          .errorhandling = "remove" ## use "pass" to see problems
+        )
+        
+        foreach_args[names(control)] <- control # replaces the above defaults by user controls
+        
+        if (showpbar) { # optionally wrap the combine function with progress bar code
+          barstyle <- eval(spaMM.getOption("barstyle"))
+          
+          .combine <- foreach_args$.combine
+          if (inherits(.combine,"character")) .combine <- get(.combine)
+          
+          progrbar_setup <- .set_progrbar(max = nsim, style = barstyle, char=pb_char)
+          combine_with_pb <- function(nsim, pb){
+            count <- 0
+            force(pb)
+            function(...) {
+              count <<- count + length(list(...))
+              setTxtProgressBar(pb, count)
+              flush.console()
+              cbind(...) # this can feed into .combine option of foreach
+            }
+          } # returns a function that increments the bar then actually combines.
+          foreach_args$.combine <- combine_with_pb(nsim, pb=progrbar_setup$pb)
+        } 
+        
+        foreach_blob <- do.call(foreach::foreach,foreach_args) 
+        
+        fn_dots <- list(...)
+        # for (st in names(fn_dots)) {
+        #   # Add an enclosing quote():
+        #   if ( is.language(fn_dots[[st]])) fn_dots[[st]] <- substitute(quote(what),list(what=fn_dots[[st]]))
+        # }
+        bootreps <- try(foreach::`%do%`(foreach_blob, do.call(fn, c(list(newresp[, i]), fn_dots)))) 
+        # the try() is useful if the user interrupts the %do%, in which case it allows close(pb) to be run.
+        
+        if (showpbar) close(progrbar_setup$pb)
+      } else { # older version using pbapply
+        if (showpbar) {
+          pbopt <- pboptions(nout=min(100L,2L*nsim),type="timer",char=pb_char) 
+        } else pbopt <- pboptions(type="none") 
+        bootreps <- pbapply(X=newresp,MARGIN = 2L,FUN = fn, cl=NULL, ...)
+        pboptions(pbopt)
+        if (identical(control$.combine,"rbind")) bootreps <- t(bootreps)
+      }
     }
     cat(paste(" bootstrap took",.timerraw(time1),"s.\n")) 
     return(bootreps)
